/*
 * about.c: About box code
 *
 * This file is part of GTick
 *
 * Copyright (c) 1999, Alex Roberts
 * Copyright (c) 2003, Roland Stigge <stigge@antcom.de>
 *
 * GTick is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * GTick is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with GTick; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 */

/* GNU headers */
#include <stdio.h>
#include <sys/ioctl.h>

/* OSS headers */
#ifdef FREEBSD
#include <machine/soundcard.h>
#else
#include <linux/soundcard.h>
#endif

/* GTK headers*/
#include <gtk/gtk.h>
#include <glib.h>

/* own headers */
#include "globals.h"
#include "metro.h"
#include "about.h"

/* logo */
#include "aboutlogo.xpm"

static void closewindow_cb (GtkWidget* w _U_, GtkWidget *window)
{
  gtk_widget_destroy (window);
}

void gtAboutBox(metro_t* metro)
{
  GtkWidget* window;
  GtkWidget* vbox;
  GtkWidget* label;

  GdkPixmap* pixmap;
  GdkBitmap* mask;
  GtkStyle* style;
  GtkWidget* pixmapwidget;

  window = gtk_dialog_new_with_buttons("About GTick", GTK_WINDOW(metro->window),
      GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
      GTK_STOCK_OK, GTK_RESPONSE_NONE,
      NULL);

  gtk_signal_connect(GTK_OBJECT(window), "destroy",
		     GTK_SIGNAL_FUNC (gtk_widget_destroy),
		     window);

  vbox = gtk_vbox_new(FALSE, 10);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), vbox, TRUE, TRUE, 0);
  gtk_widget_show(vbox);

  gtk_widget_realize(window);
  style = gtk_widget_get_style(window);
  pixmap = gdk_pixmap_create_from_xpm_d(window->window,
  				        &mask,
				        NULL,
				        (gchar**) aboutlogo_xpm);
  pixmapwidget = gtk_pixmap_new(pixmap, mask);
  gtk_box_pack_start(GTK_BOX(vbox), pixmapwidget, TRUE, TRUE, 0);
  gtk_widget_show(pixmapwidget);
  
  label = gtk_label_new (_(
    "Copyright (c) 1999, Alex Roberts <bse@dial.pipex.com>\n"
    "Copyright (c) 2003, Roland Stigge <stigge@antcom.de>\n"
    "Logo by Mario Andres Pagella <mapagella@fitzroy-is.com.ar>\n"
    // XXX:Andrs
    "\n"
    "Homepage:\n"
    "http://www.antcom.de/gtick/\n"
    "\n"
    "Email bug reports, comments, etc. to\n"
    "Developer's mailing list:\n"
    "gtick-devel@nongnu.org"
  ));
  gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_CENTER);
  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
  gtk_widget_show(label);

  g_signal_connect_swapped(GTK_OBJECT(window),
                           "response",
			   G_CALLBACK(gtk_widget_destroy),
			   GTK_OBJECT(window));
  gtk_widget_show(window);
}

